@@ -6,6 +6,7 @@
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution.
 
+import operator
 import sys
 import jsonpickle.util as util
 import jsonpickle.tags as tags
@@ -22,11 +23,17 @@ class Unpickler(object):
         ## The namestack grows whenever we recurse into a child object
         self._namestack = []
 
+        ## Maps objects to their index in the _objs list
+        self._obj_to_idx = {}
+        self._objs = []
+
     def _reset(self):
         """Resets the object's internal state.
         """
         self._namedict = {}
         self._namestack = []
+        self._obj_to_idx = {}
+        self._objs = []
 
     def _push(self):
         """Steps down one level in the namespace.
@@ -55,6 +62,9 @@ class Unpickler(object):
         """
         self._push()
 
+        if has_tag(obj, tags.ID):
+            return self._pop(self._objs[obj[tags.ID]])
+
         if has_tag(obj, tags.REF):
             return self._pop(self._namedict.get(obj[tags.REF]))
 
@@ -80,7 +90,10 @@ class Unpickler(object):
                 return self._pop(handler.restore(obj))
 
             try:
-                instance = object.__new__(cls)
+                if hasattr(cls, '__new__'):
+                    instance = cls.__new__(cls)
+                else:
+                    instance = object.__new__(cls)
             except TypeError:
                 # old-style classes
                 try:
@@ -98,7 +111,7 @@ class Unpickler(object):
                 instance.__setstate__(state)
                 return self._pop(instance)
 
-            for k, v in obj.iteritems():
+            for k, v in sorted(obj.iteritems(), key=operator.itemgetter(0)):
                 # ignore the reserved attribute
                 if k in tags.RESERVED:
                     continue
@@ -135,7 +148,7 @@ class Unpickler(object):
 
         if util.is_dictionary(obj):
             data = {}
-            for k, v in obj.iteritems():
+            for k, v in sorted(obj.iteritems(), key=operator.itemgetter(0)):
                 self._namestack.append(k)
                 data[k] = self.restore(v)
                 self._namestack.pop()
@@ -175,15 +188,18 @@ class Unpickler(object):
         >>> thing = Thing('referenced-thing')
         >>> u = Unpickler()
         >>> u._mkref(thing)
-        '/'
-        >>> u._namedict['/']
+        0
+        >>> u._objs[0]
         samples.Thing("referenced-thing")
 
         """
-        name = self._refname()
-        if name not in self._namedict:
-            self._namedict[name] = obj
-        return name
+        obj_id = id(obj)
+        try:
+            idx = self._obj_to_idx[obj_id]
+        except KeyError:
+            idx = self._obj_to_idx[obj_id] = len(self._objs)
+            self._objs.append(obj)
+        return idx
 
 def loadclass(module_and_name):
     """Loads the module and returns the class.
