@@ -9,24 +9,30 @@
 
 import os
 import sys
-sys.path.append(os.path.join(os.path.dirname(__file__), '..'))
+
+testdir = os.path.dirname(os.path.abspath(__file__))
+sys.path.insert(1, os.path.dirname(testdir))
 
 import unittest
 
 import util_tests
 import jsonpickle_test
 import thirdparty_tests
+import backends_tests
+import document_test
 
 def suite():
     suite = unittest.TestSuite()
     suite.addTest(util_tests.suite())
     suite.addTest(jsonpickle_test.suite())
+    suite.addTest(document_test.suite())
     suite.addTest(thirdparty_tests.suite())
+    suite.addTest(backends_tests.suite())
     return suite
 
 def main():
     #unittest.main(defaultTest='suite')
-    unittest.TextTestRunner(verbosity=2).run(suite())
+    return unittest.TextTestRunner(verbosity=2).run(suite())
 
 if __name__ == '__main__':
-    main()
+    sys.exit(not main().wasSuccessful())
